package rpc

import (
	"context"
	"fmt"

	"git.loafle.net/commons_go/logging"
	"git.loafle.net/commons_go/rpc"
	crcwf "git.loafle.net/commons_go/rpc/connection/websocket/fasthttp"
	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol"
	"git.loafle.net/commons_go/rpc/protocol/json"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
	"google.golang.org/grpc/metadata"
)

func newGatewayRPCServletHandler() GatewayRPCServletHandler {
	sh := &GatewayRPCServletHandlers{}
	sh.RegisterCodec(json.NewServerCodec(), json.Name)
	return sh
}

type GatewayRPCServletHandlers struct {
	gateway.ServletHandlers
	crcwf.ServletHandlers
}

func (sh *GatewayRPCServletHandlers) Invoke(servletCTX rpc.ServletContext, requestCodec protocol.RegistryCodec) (result interface{}, err error) {
	if !sh.RPCRegistry.HasMethod(requestCodec.Method()) {
		return nil, fmt.Errorf("RPC Servlet Handler: Method[%s] is not exist", requestCodec.Method())
	}

	md := metadata.Pairs(oogw.GRPCUserIDKey, soc.ID())
	grpcCTX := metadata.NewOutgoingContext(context.Background(), md)

	r, err := grpc.Exec(grpcCTX, requestCodec.Method(), params)
	if err != nil {
		return nil, err
	}

	return r, nil
}

func (sh *GatewayRPCServletHandlers) Validate() {
	sh.ServletHandlers.Validate()

	if nil == sh.RPCRegistry {
		logging.Logger().Panic(fmt.Sprintf("RPC Servlet Handler: RPC Registry must be specified"))
	}
}
